# -*- coding: utf-8 -*-

from PyQt5 import QtCore
from PyQt5.QtCore import Qt

from PyQt5 import QtGui
from PyQt5 import QtWidgets

from PyQt5 import QtNetwork

#from PyQt5 import QtWebSockets
#from PyQt5 import QtWebKit
#from PyQt5 import QtWebKitWidgets
#from PyQt5 import QtSql


from PyQt5.QtCore import pyqtSignal

